''' <summary>
''' Contains informations about the output of a building.
''' </summary>
Public Class BuildingOutput
    ''' <summary>
    ''' Boolean value that indicates if the building has been succesful.
    ''' </summary>
    Public ok As Boolean = True
    ''' <summary>
    ''' Build log.
    ''' </summary>
    Public log As New buildLog
    Private _errors As buildError()
    ''' <summary>
    ''' Descriptions of the building errors.
    ''' </summary>
    Public Property errors() As buildError()
        Get
            Return _errors
        End Get
        Set(ByVal value As buildError())
            If Not Object.Equals(value, Nothing) Then
                If value.Length = 0 Then
                    ok = True
                Else
                    ok = False
                End If
                _errors = value
            End If
        End Set
    End Property
End Class

''' <summary>
''' Contains informations about a building error.
''' </summary>
Public Structure buildError
    ''' <summary>
    ''' Error's description.
    ''' </summary>
    Public descr As String
    ''' <summary>
    ''' Details about the file that contains the error.
    ''' </summary>
    Public file As fileDetails
    ''' <summary>
    ''' Error's line in the file.
    ''' </summary>
    Public line As Integer
    ''' <summary>
    ''' Error's column in the file.
    ''' </summary>
    Public column As Integer
    ''' <summary>
    ''' Initializes a new istance of the structure error.
    ''' </summary>
    ''' <param name="d">Error's description</param>
    ''' <param name="f">Details about the file that contains the error.</param>
    ''' <param name="l">Error's line in the file.</param>
    ''' <param name="c">Error's column in the file.</param>
    Sub New(ByVal d As String, ByVal f As fileDetails, ByVal l As Integer, ByVal c As Integer)
        descr = d
        file = f
        line = l
        column = c
    End Sub
End Structure

''' <summary>
''' Represents the log of a building.
''' </summary>
Public NotInheritable Class buildLog
    Private actions As New List(Of String)
    Private times As New List(Of DateTime)
    Public Event actionAdded(ByVal time As DateTime, ByVal action As String)

    Public Overrides Function ToString() As String
        Dim s As String = ""
        For i As Integer = 0 To actions.Count - 1
            s &= "[" & times(i).ToLongTimeString() & "] " & actions(i) & Environment.NewLine
        Next
        Return s
    End Function

    ''' <summary>
    ''' Adds an action to the log.
    ''' </summary>
    ''' <param name="descr">Description of the done action.</param>
    Public Sub add(ByVal descr As String)
        actions.Add(descr)
        times.Add(DateTime.Now)
        RaiseEvent actionAdded(DateTime.Now, descr)
    End Sub
End Class