Imports System.Windows.Forms
Imports System.Drawing
Imports LangLib

Public Class CodeEditor
    Inherits Editor
    WithEvents rtb As New RichTextBox
    Dim fileExt As String
    Dim langs As Language()
    Dim mainLang As Language


    Sub New(ByVal ext As String, ByVal lngs As Language())
        rtb.Dock = DockStyle.Fill
        rtb.WordWrap = False
        rtb.Font = New Font("Courier New", 11)
        rtb.AcceptsTab = True
        Me.Controls.Add(rtb)
        fileExt = ext
        langs = lngs
        Dim flngs As New List(Of Language)
        For Each l As Language In langs
            For Each p As LanguagePosition In l.pos
                If p = New LanguagePosition_FullFile(ext) Then
                    flngs.Add(l)
                    Exit For
                End If
            Next
        Next
        If flngs.Count = 0 Then
            MsgBox(getMsg("editorNoLanguage", ext), MsgBoxStyle.Exclamation)
            mainLang = Nothing
        ElseIf flngs.Count > 1 Then
            MsgBox(getMsg("editorTooManyLanguages", ext), MsgBoxStyle.Exclamation)
            Dim i As Integer = 0
            Do
                Dim cln As Language = flngs(i)
                If MsgBox(getMsg("editorUseLanguage", cln.name), MsgBoxStyle.YesNo) = MsgBoxResult.Yes Then
                    mainLang = cln
                End If
                i += 1
            Loop Until Not Object.Equals(mainLang, Nothing)
        ElseIf flngs.Count = 1 Then
            mainLang = flngs(0)
        End If
        Dim lng As New List(Of Language)(langs)
        lng.RemoveAll(New Predicate(Of Language)(AddressOf valLn))
        langs = lng.ToArray()
    End Sub

    Function valLn(ByVal obj As Language) As Boolean
        For Each p As LanguagePosition In obj.pos
            If p.GetType.Equals(GetType(LanguagePosition_InOtherLanguage)) Then
                If CType(p, LanguagePosition_InOtherLanguage).container = mainLang.name Then
                    Return False
                End If
            End If
        Next
        Return True
    End Function

    Public Overrides Sub open(ByVal file As String)
        Dim s As String = ""
        Try
            s = My.Computer.FileSystem.ReadAllText(file)
        Catch ex As Exception
            MsgBox(ex.Message)
        End Try
        rtb.Text = s
        highlight(True)
    End Sub

    Public Overrides Sub cut()
        rtb.Cut()
    End Sub

    Public Overrides Sub copy()
        rtb.Copy()
    End Sub

    Public Overrides Sub paste()
        rtb.Paste()
        highlight(True)
    End Sub

    Public Overrides Sub undo()
        rtb.Undo()
    End Sub

    Public Overrides Sub redo()
        rtb.Redo()
    End Sub

    Public Overrides Sub selectAll()
        rtb.SelectAll()
    End Sub

    Public Overrides Sub save(ByVal file As String)
        Try
            My.Computer.FileSystem.WriteAllText(file, rtb.Text, False)
        Catch ex As Exception
            MsgBox(ex.Message)
        End Try
    End Sub

    Public Sub highlight(ByVal p As Boolean)
        Dim selS As Integer = rtb.SelectionStart
        Dim selL As Integer = rtb.SelectionLength
        Dim f As Integer = 0
        Dim t As Integer = rtb.Text.Length
        Dim lang As Language = mainLang
        For Each cl As Language In langs
            For Each cp As LanguagePosition_InOtherLanguage In cl.pos
                Dim b() As Char = rtb.Text.Substring(0, rtb.SelectionStart).ToCharArray()
                Dim a() As Char = rtb.Text.Substring(rtb.SelectionStart).ToCharArray()
                Dim bc As Integer = 0
                Dim ac As Integer = 0
                If b.Length <> 0 Then
                    For i As Integer = 0 To b.Length - 1
                        If b(i) = cp.beginStr Then
                            bc += 1
                        End If
                    Next
                End If
                If a.Length <> 0 Then
                    For i As Integer = 0 To a.Length - 1
                        If a(i) = cp.endStr Then
                            ac += 1
                        End If
                    Next
                End If
                If cp.beginStr = cp.endStr Then
                    If bc Mod 2 <> 0 Then
                        If rtb.Lines(rtb.GetLineFromCharIndex(rtb.SelectionStart)).Contains(cp.beginStr) Then
                            f = rtb.GetFirstCharIndexOfCurrentLine + rtb.Lines(rtb.GetLineFromCharIndex(rtb.SelectionStart)).IndexOf(cp.beginStr)
                        End If
                        If rtb.Lines(rtb.GetLineFromCharIndex(rtb.SelectionStart)).Contains(cp.endStr) Then
                            t = rtb.GetFirstCharIndexOfCurrentLine + rtb.Lines(rtb.GetLineFromCharIndex(rtb.SelectionStart)).LastIndexOf(cp.endStr)
                        End If
                        lang = cl
                    End If
                Else

                End If
            Next
        Next
        lang.textEdited(rtb, p, f, t)
        rtb.SelectionStart = selS
        rtb.SelectionLength = selL
    End Sub

    Private Sub rtb_KeyPress(ByVal sender As Object, ByVal e As KeyEventArgs) Handles rtb.KeyUp
        If e.Control And e.KeyCode = Keys.V Then
            highlight(True)
        Else
            highlight(False)
        End If
    End Sub
End Class
