﻿Imports System.Windows.Forms
Imports LangLib
Imports System.Net
Imports System.Threading

''' <summary>
''' Tool used in UniversalIDE to download files, could be useful also for add-ons.
''' </summary>
Public Class DownloaderTool
    Dim items As New List(Of itemToDownload)
    WithEvents w As New WebClient
    Dim index As Integer = -1
    Dim totalLabel, currentLabel As String

    Public Shared Sub download(ByVal itemsToDownload As List(Of itemToDownload))
        Dim dt As New DownloaderTool
        dt.items = itemsToDownload
        dt.ShowDialog()
    End Sub

    Private Sub DownloaderTool_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        applyLang(Me)
        totalLabel = lblTotal.Text
        currentLabel = lblCurrent.Text
        For Each i As itemToDownload In items
            itemList.Items.Add(i.name)
        Next
        Control.CheckForIllegalCrossThreadCalls = False
        Dim th As New Thread(AddressOf nextDownload)
        th.Start()
    End Sub

    Private Sub btnCancel_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCancel.Click
        cancel()
        Me.DialogResult = System.Windows.Forms.DialogResult.Cancel
        Me.Close()
    End Sub

    Sub nextDownload()
        index += 1
        lblTotal.Text = String.Format("{0} ({1}/{2})", totalLabel, index, items.Count)
        If index = items.Count Then
            Me.DialogResult = System.Windows.Forms.DialogResult.OK
            Me.Close()
        Else
            Dim i As itemToDownload = items(index)
            If (index > 0) Then itemList.Items(index - 1) = CStr(itemList.Items(index - 1)).Remove(0, 2)
            itemList.Items(index) = "> " & itemList.Items(index)
            w.DownloadFileAsync(New Uri(i.url), i.path)
        End If
    End Sub

    Sub cancel()
        If MsgBox(getMsg("cancelDownloadConfirm"), MsgBoxStyle.YesNo, "UniversalIDE") = MsgBoxResult.Yes Then
            w.CancelAsync()
            For Each i As itemToDownload In items
                Try
                    My.Computer.FileSystem.DeleteFile(i.path)
                Catch ex As Exception
                End Try
            Next
        End If
    End Sub

    Private Sub w_ProgressChanged(ByVal sender As Object, ByVal e As DownloadProgressChangedEventArgs) Handles w.DownloadProgressChanged
        currentProgress.Value = e.ProgressPercentage
        lblCurrent.Text = String.Format("{0} ({1}%)", currentLabel, e.ProgressPercentage)
    End Sub

    Private Sub w_DownloadFileCompleted(ByVal sender As Object, ByVal e As EventArgs) Handles w.DownloadFileCompleted
        totalProgress.Value = index
        nextDownload()
    End Sub
End Class

Public Structure itemToDownload
    Public url As String
    Public path As String
    Public name As String

    Sub New(ByVal u As String, ByVal p As String, n As String)
        url = u
        path = p
        name = n
    End Sub
End Structure
