Imports System.Windows.Forms

Public Module LangUtil
    Public currLang As Lang
    Dim lngDef As controlsDef

    Sub loadFiles(ByVal lng As String, ByVal def As String)
        currLang = Lang.load(lng)
        lngDef = controlsDef.load(def)
    End Sub

    Private Function getControls(ByRef parent As Control) As List(Of Control)
        Dim l As New List(Of Control)
        For Each c As Control In parent.Controls
            l.Add(c)
            l.AddRange(getControls(c))
        Next
        Return l
    End Function

    Private Function getMenus(ByVal controls As List(Of Control)) As List(Of ToolStripItem)
        Dim l As New List(Of ToolStripItem)
        For Each c As Control In controls
            Dim m As ToolStrip = TryCast(c, ToolStrip)
            If Not Object.Equals(m, Nothing) Then
                For Each i As ToolStripItem In m.Items
                    l.Add(i)
                    Dim dropDown As ToolStripDropDownItem = TryCast(i, ToolStripDropDownItem)
                    If Not Object.Equals(dropDown, Nothing) Then
                        l.AddRange(getSubMenus(dropDown))
                    End If
                Next
            End If
        Next
        Return l
    End Function

    Private Function getSubMenus(ByVal parent As ToolStripDropDownItem) As List(Of ToolStripItem)
        Dim l As New List(Of ToolStripItem)
        For Each m As ToolStripItem In parent.DropDownItems
            l.Add(m)
            Try
                l.AddRange(getSubMenus(m))
            Catch ex As Exception
            End Try
        Next
        Return l
    End Function

    Public Sub applyLang(ByRef frm As Form, ByVal ParamArray otherControl() As Control)
        Try
            frm.Text = currLang.getFormTitle(frm.Name)
            Dim cList As List(Of Control) = getControls(frm)
            cList.AddRange(otherControl)
            'ONLY FOR DEBUG

            'Dim s As String = ""
            'For Each c As Control In cList
            '    s &= c.Name & Chr(13)
            'Next
            'MsgBox(s)
            's = ""
            'For Each m As ToolStripItem In getMenus(cList)
            '    s &= m.Name & Chr(13)
            'Next
            'MsgBox(s)

            'END
            For Each c As Control In cList
                Try
                    If TypeOf c Is ListView Then
                        For Each column As ColumnHeader In CType(c, ListView).Columns
                            column.Text = currLang.getControlText(frm.Name, column.Text)
                        Next
                    End If
                    c.Text = currLang.getControlText(frm.Name, c.Name)
                    Dim af As String = lngDef.tryGetAssociatedField(frm.Name, c.Name)
                    If af <> "" Then
                        For Each field As Control In cList
                            If field.Name = af Then
                                Dim startPos As Integer = c.Location.X + c.Width + 6
                                Dim maxWidth As Integer = field.Location.X + field.Width
                                field.Width = maxWidth - startPos
                                Dim newPos As Drawing.Point = field.Location
                                newPos.X = startPos
                                field.Location = newPos
                            End If
                        Next
                    End If
                Catch ex As langException
                    If ex.errorType = langErrType.ControlNotFound Then
                        If (Object.Equals(c.GetType(), GetType(Label)) Or Object.Equals(c.GetType(), GetType(Button))) And (Not lngDef.ignorableControls.Contains(New ignorableControl(frm.Name, c.Name))) Then
                            ex.showMsg()
                        End If
                    End If
                End Try
            Next
            For Each m As ToolStripItem In getMenus(cList)
                Try
                    Dim mt As menuText = currLang.getMenuData(frm.Name, m.Name)
                    m.Text = mt.text
                    m.ToolTipText = mt.tooltip
                Catch ex As langException
                    If ex.errorType = langErrType.MenuNotFound Then
                        If Not (Object.Equals(m.GetType(), GetType(ToolStripSeparator)) Or Object.Equals(m.GetType(), GetType(ToolStripStatusLabel)) Or Object.Equals(m.GetType(), GetType(ToolStripProgressBar))) Then
                            If m.Tag <> "notranslation" Then
                                ex.showMsg()
                            End If
                        End If
                    End If
                End Try
            Next
        Catch ex As langException
            ex.showMsg()
        End Try
    End Sub

    Public Sub applylangControl(ByVal frmName As String, ByRef ctrl As Control)
        Dim cList As List(Of Control) = getControls(ctrl)
        For Each c As Control In cList
            Try
                c.Text = currLang.getControlText(frmName, c.Name)
                Dim af As String = lngDef.tryGetAssociatedField(frmName, c.Name)
                If af <> "" Then
                    For Each field As Control In cList
                        If field.Name = af Then
                            Dim startPos As Integer = c.Location.X + c.Width + 6
                            Dim maxWidth As Integer = field.Location.X + field.Width
                            field.Width = maxWidth - startPos
                            Dim newPos As Drawing.Point = field.Location
                            newPos.X = startPos
                            field.Location = newPos
                        End If
                    Next
                End If
            Catch ex As langException
                If ex.errorType = langErrType.ControlNotFound Then
                    If (Object.Equals(c.GetType(), GetType(Label)) Or Object.Equals(c.GetType(), GetType(Button))) And (Not lngDef.ignorableControls.Contains(New ignorableControl(frmName, c.Name))) Then
                        ex.showMsg()
                    End If
                End If
            End Try
        Next
    End Sub

    Public Function getMsg(ByVal id As String, ByVal ParamArray args() As String) As String
        Try
            Dim foundMessages As New List(Of guiMessage)
            For Each m As guiMessage In currLang.guiMessages
                If m.id = id Then
                    foundMessages.Add(m)
                End If
            Next
            If foundMessages.Count = 0 Then
                Throw New langException(currLang, langErrType.MessageNotFound, id)
            ElseIf foundMessages.Count > 1 Then
                Throw New langException(currLang, langErrType.MessageConflict, id)
            Else
                Return String.Format(foundMessages(0).msg, args)
            End If
        Catch ex As langException
            ex.showMsg()
            Return Nothing
        End Try
    End Function
End Module
