Imports AddOnsLib
Imports LangLib
Imports System.Reflection

Public Class AddonManager
    Public installedAddons As New List(Of addonManifest)
    Private t As Type()

    Sub New(Optional ByVal noAddons As Boolean = False)
        installedAddons = getInstalledAddons()
        If Not noAddons Then t = loadTypes()
    End Sub

    Public ReadOnly Property isInNoAddonsMode As Boolean
        Get
            Return IsNothing(t)
        End Get
    End Property

    Private Function getInstalledAddons() As List(Of addonManifest)
        Dim result As New List(Of addonManifest)
        For Each d As String In My.Computer.FileSystem.GetDirectories(Application.StartupPath & "\Addons\")
            result.Add(addonManifest.load(d & "\manifest.xml"))
            Dim settingsDir As String = MainForm.settingsPath & "\" & result(result.Count - 1).shortName
            If Not My.Computer.FileSystem.DirectoryExists(settingsDir) Then
                My.Computer.FileSystem.CreateDirectory(settingsDir)
            End If
        Next
        Return result
    End Function

    Private Function loadTypes() As Type()
        Dim l As New List(Of Type)
        For Each mf As addonManifest In installedAddons
            Dim d As String = Application.StartupPath & "\Addons\" & mf.shortName
            Try
                Dim f As String = d & "\" & mf.assemblyFileName
                If My.Computer.FileSystem.FileExists(f) Then
                    Dim a As Assembly = Assembly.LoadFrom(f)
                    l.AddRange(a.GetTypes())
                Else
                    MsgBox(getMsg("addonDirError", mf.fullName), MsgBoxStyle.Critical)
                End If
            Catch ex As Exception
                MsgBox(getMsg("noAddonManifest", d), MsgBoxStyle.Critical)
            End Try
        Next
        Return l.ToArray()
    End Function

    Public Function getLanguages() As Language()
        Dim l As New List(Of Language)
        For Each i As Type In t
            If i.BaseType Is GetType(Language) Then
                l.Add(CType(Activator.CreateInstance(i), Language))
            End If
        Next
        Return l.ToArray()
    End Function

    Public Function getAppTypes() As ApplicationType()
        Dim l As New List(Of ApplicationType)
        For Each i As Type In t
            If i.BaseType.BaseType Is GetType(ApplicationType) Then
                l.Add(CType(Activator.CreateInstance(i), ApplicationType))
            End If
        Next
        Return l.ToArray()
    End Function

    Public Function getLanguage(ByVal name As String) As Language
        Dim l As New List(Of Language)
        For Each i As Language In getLanguages()
            If i.name = name Then
                l.Add(i)
            End If
        Next
        If l.Count = 1 Then
            Return l(0)
        Else
            Throw New ArgumentException(getMsg("languageNotFound"))
        End If
    End Function

    Public Function getAppType(ByVal name As String) As ApplicationType
        Dim l As New List(Of ApplicationType)
        For Each i As ApplicationType In getAppTypes()
            If i.name = name Then
                l.Add(i)
            End If
        Next
        If l.Count = 1 Then
            Return l(0)
        Else
            Throw New ArgumentException(getMsg("appTypeNotFound", name))
        End If
    End Function

    Public Function exists(ByVal t As itemType, ByVal n As String) As Boolean
        Try
            If t = itemType.Language Then
                getLanguage(n)
            ElseIf t = itemType.AppType Then
                getAppType(n)
            End If
            Return True
        Catch ex As Exception
            Return False
        End Try
    End Function
End Class

Public Enum itemType
    Language
    AppType
End Enum