﻿Imports LangLib
Imports System.Windows.Forms
Imports ICSharpCode.SharpZipLib.Zip
Imports System.Threading
Imports System.Net

Public Class AddonManagerDialog

    Private Class installStatus
        Enum installStatusValue
            PreparingDownload
            Downloading
            Extracting
            DownloadingLang
        End Enum
        Public shortName As String
        Public fullName As String
        Public zipPath As String
        Public status As installStatusValue
        Public dlProgress As Integer = 0
        Sub New(ByVal sn As String, ByVal fn As String, ByVal z As String, ByVal s As installStatusValue)
            shortName = sn
            fullName = fn
            zipPath = z
            status = s
        End Sub
    End Class

    Dim baseDir As String = Application.StartupPath & "\Addons"
    Dim uninstall As New List(Of String)
    Dim updates As New List(Of String)
    Dim installing As New List(Of installStatus)

#Region "Event handlers"
    Private Sub AddonManagerDialog_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        applyLang(Me)
        Control.CheckForIllegalCrossThreadCalls = False
        itemName.Text = getMsg("itemName")
        itemStatus.Text = getMsg("itemStatus")
        refreshList()
        Dim updaterThread As New Thread(Sub()
                                            Try
                                                Dim updatesData() As String = RepositoryApi.checkAddonsUpdates(MainForm.addons.installedAddons)
                                                For Each i As String In updatesData
                                                    updates.Add(i)
                                                Next
                                                refreshList()
                                            Catch ex As Exception
                                            End Try
                                        End Sub)
        updaterThread.Start()

    End Sub

    Private Sub btnClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClose.Click
        Me.DialogResult = System.Windows.Forms.DialogResult.OK
        Me.Close()
    End Sub

    Private Sub btnUpdate_Click(sender As Object, e As EventArgs) Handles btnUpdate.Click
        If isSelected() Then
            If updates.Contains(addonList.SelectedItems(0).Tag) Then
                If checkNoAddonsMode() Then
                    My.Computer.FileSystem.DeleteDirectory(Application.StartupPath & "\Addons\" & addonList.SelectedItems(0).Tag, FileIO.DeleteDirectoryOption.DeleteAllContents)
                    downloadAndInstall(RepositoryApi.getAddonManifest(addonList.SelectedItems(0).Tag))
                End If
            Else
                MsgBox(getMsg("addonAlreadyUpToDate"), MsgBoxStyle.Information)
            End If
        End If
    End Sub

    Private Sub btnUpdateAll_Click(sender As Object, e As EventArgs) Handles btnUpdateAll.Click
        MsgBox(getMsg("notImplemented"))
    End Sub

    Private Sub btnConfigure_Click(sender As Object, e As EventArgs) Handles btnConfigure.Click
        MsgBox(getMsg("notImplemented"))
    End Sub

    Private Sub btnUninstall_Click(sender As Object, e As EventArgs) Handles btnUninstall.Click
        If isSelected() Then
            Dim n As String = addonList.SelectedItems(0).Tag
            If Not uninstall.Contains(n) Then
                uninstall.Add(n)
            Else
                uninstall.Remove(n)
            End If
            refreshList()
        End If
    End Sub

    Private Sub btnInstallFromFile_Click(sender As Object, e As EventArgs) Handles btnInstallFromFile.Click
        If installFromFileDialog.ShowDialog() = Windows.Forms.DialogResult.OK Then
            Dim fInfo As New IO.FileInfo(installFromFileDialog.FileName)
            Dim mf As addonManifest
            Try
                mf = addonManifest.load(installFromFileDialog.FileName)
            Catch ex As Exception
                MsgBox(getMsg("invalidManifest"), MsgBoxStyle.Critical)
                Exit Sub
            End Try
            downloadAndInstall(mf)
            refreshList()
        End If
    End Sub

    Private Sub btnInstallFromRepo_Click(sender As Object, e As EventArgs) Handles btnInstallFromRepo.Click
        Dim re As New RepoExplorer
        If re.ShowDialog() = Windows.Forms.DialogResult.Cancel Then Exit Sub
        downloadAndInstall(re.selectedAddonMF)
    End Sub

    Private Sub AddonManagerDialog_FormClosing(sender As Object, e As FormClosingEventArgs) Handles MyBase.FormClosing
        If uninstall.Count > 0 Then
            Dim s As String = ""
            For i As Integer = 0 To uninstall.Count - 1
                s &= uninstall(i)
                If i <> uninstall.Count - 1 Then s &= Environment.NewLine
            Next
            My.Computer.FileSystem.WriteAllText(baseDir & "\uninstall.txt", s, False)
        End If
    End Sub

    Private Sub btnRestart_Click(sender As Object, e As EventArgs) Handles btnRestart.Click
        Application.Restart()
    End Sub
#End Region

#Region "Functions"
    Sub refreshList()
        addonList.Items.Clear()
        For Each d As String In My.Computer.FileSystem.GetDirectories(baseDir)
            Try
                Dim mf As addonManifest = addonManifest.load(d & "\manifest.xml")
                Dim status As String = getMsg("statusUpdated")
                If Not My.Computer.FileSystem.DirectoryExists(MainForm.settingsPath & "\" & mf.shortName) Then status = getMsg("statusInstall")
                If updates.Contains(mf.shortName) Then status = getMsg("statusUpdateAvaiable")
                If uninstall.Contains(mf.shortName) Then status = getMsg("statusUninstall")
                Dim lvi As New ListViewItem({mf.fullName, status})
                lvi.Tag = mf.shortName
                addonList.Items.Add(lvi)
            Catch ex As Exception
            End Try
        Next
        For Each item As installStatus In installing
            Dim lvi As New ListViewItem({item.fullName, getMsg(item.status.ToString())})
            Dim statusString As String = ""
            If item.status = installStatus.installStatusValue.Downloading Then lvi.SubItems(1).Text = item.dlProgress & "% " & lvi.SubItems(1).Text
            lvi.Tag = item.shortName
            addonList.Items.Add(lvi)
        Next
    End Sub

    Private Sub refreshListItem(ByVal status As installStatus)
        For Each i As ListViewItem In addonList.Items
            If i.Text = status.fullName Then
                Dim s As String = ""
                If Not installing.Contains(status) Then
                    s = getMsg("statusUpdated")
                    If Not My.Computer.FileSystem.DirectoryExists(MainForm.settingsPath & "\" & i.Tag) Then s = getMsg("statusInstall")
                Else
                    Dim progress As String = ""
                    If status.status = installStatus.installStatusValue.Downloading Then progress = status.dlProgress & "% "
                    s = progress & getMsg(status.status.ToString())
                End If
                i.SubItems(1).Text = s
                Exit Sub
            End If
        Next
    End Sub

    Function isSelected() As Boolean
        If addonList.SelectedIndices.Count <> 0 Then
            Return True
        Else
            MsgBox(getMsg("noAddOnSelected"), MsgBoxStyle.Critical)
            Return False
        End If
    End Function

    Function checkNoAddonsMode() As Boolean
        If MainForm.addons.isInNoAddonsMode Then
            Return True
        Else
            If MessageBox.Show(getMsg("requiredNoAddonsMode"), "UniversalIDE", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) = Windows.Forms.DialogResult.Yes Then
                Process.Start(Application.ExecutablePath, "-noaddons")
                Application.Exit()
            End If
            Return False
        End If
    End Function

    Function solveDependancies(ByVal mf As addonManifest) As List(Of addonManifest)
        Dim deps As addonDependancySolverResult = addonDependancySolver.solveDependancies(mf)
        Dim msg As String = Environment.NewLine
        Dim toInstall As New List(Of addonManifest)
        If checkCompatibility(mf) Then
            toInstall.Add(mf)
        Else
            Return Nothing
        End If
        Dim fullNames As Dictionary(Of String, String) = RepositoryApi.getFullNames()
        msg &= "- " & mf.fullName & " " & mf.version & Environment.NewLine
        For i As Integer = 0 To deps.dep.Count - 1
            Dim install As Boolean = False
            Dim fullName As String = fullNames(deps.dep(i).addonName)
            Select Case deps.depStatus(i)
                Case dependancyStatus.missing
                    install = True
                Case dependancyStatus.unknownSupport
                    If MessageBox.Show(getMsg("unknownSupport", fullName, mf.fullName), "UniversalIDE", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) = Windows.Forms.DialogResult.Yes Then install = True
                Case dependancyStatus.versionUnsupported
                    If MessageBox.Show(getMsg("versionUnsupported", fullName, mf.fullName), "UniversalIDE", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) = Windows.Forms.DialogResult.Yes Then install = True
            End Select
            Dim depmf As addonManifest = RepositoryApi.getAddonManifest(deps.dep(i).addonName)
            If Not checkCompatibility(depmf) Then Return Nothing
            If install Then
                msg &= "- " & fullName & Environment.NewLine
                toInstall.Add(depmf)
            End If
        Next
        If MsgBox(getMsg("addonInstallMessage", msg), MsgBoxStyle.YesNo) = MsgBoxResult.Yes Then
            Return toInstall
        Else
            Return Nothing
        End If
    End Function

    Function checkCompatibility(ByVal mf As addonManifest) As Boolean
        Select Case mf.checkCompatibility()
            Case compatibilityLevel.yes
                Return True
            Case compatibilityLevel.no
                If MessageBox.Show(getMsg("uncompatible", mf.fullName), "UniversalIDE", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) = Windows.Forms.DialogResult.Yes Then Return True
            Case compatibilityLevel.unknown
                If MessageBox.Show(getMsg("unknownCompatibility", mf.fullName), "UniversalIDE", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) = Windows.Forms.DialogResult.Yes Then Return True
        End Select
        Return False
    End Function

    Sub downloadAndInstall(ByVal manifest As addonManifest)
        Dim toInstall = solveDependancies(manifest)
        If IsNothing(toInstall) Then Exit Sub
        For Each mf As addonManifest In toInstall
            Dim zipPath As String = baseDir & "\" & mf.shortName & ".zip"
            Dim status As New installStatus(mf.shortName, mf.fullName, zipPath, installStatus.installStatusValue.PreparingDownload)
            installing.Add(status)
            refreshList()
            Dim downloadTh As New Thread(Sub()
                                             Dim url As String = RepositoryApi.getSourceforgeDownloadLink(RepositoryApi.getAddonDownloadLink(mf.shortName, mf.version))
                                             status.status = installStatus.installStatusValue.Downloading
                                             Dim w As New WebClient
                                             w.DownloadFileAsync(New Uri(url), zipPath)
                                             AddHandler w.DownloadProgressChanged, Sub(wSender As Object, wE As DownloadProgressChangedEventArgs)
                                                                                       status.dlProgress = wE.ProgressPercentage
                                                                                       refreshListItem(status)
                                                                                   End Sub
                                             AddHandler w.DownloadFileCompleted, Sub()
                                                                                     status.status = installStatus.installStatusValue.Extracting
                                                                                     install(zipPath, mf)
                                                                                     status.status = installStatus.installStatusValue.DownloadingLang
                                                                                     refreshListItem(status)
                                                                                     Dim langW As New WebClient
                                                                                     Try
                                                                                         My.Computer.FileSystem.CreateDirectory(baseDir & "\" & mf.shortName & "\lang")
                                                                                         langW.DownloadFileAsync(New Uri(RepositoryApi.getSourceforgeDownloadLink(RepositoryApi.getAddonTranslationDownloadLink(mf.shortName, mf.version, MainForm.mainSettings.getItemValue("lang")))), baseDir & "\" & mf.shortName & "\lang\" & MainForm.mainSettings.getItemValue("lang") & ".xml")
                                                                                         AddHandler langW.DownloadFileCompleted, Sub()
                                                                                                                                     installing.Remove(status)
                                                                                                                                     refreshListItem(status)
                                                                                                                                 End Sub
                                                                                     Catch ex As Exception
                                                                                         installing.Remove(status)
                                                                                         refreshListItem(status)
                                                                                     End Try
                                                                                     My.Computer.FileSystem.DeleteFile(zipPath)
                                                                                 End Sub
                                         End Sub)
            downloadTh.Start()
        Next
    End Sub

    Sub install(ByVal zipPath As String, ByVal mf As addonManifest)
        Dim fz As New FastZip()
        Dim currDir As String = baseDir & "\" & mf.shortName
        If My.Computer.FileSystem.DirectoryExists(currDir) Then My.Computer.FileSystem.DeleteDirectory(currDir, FileIO.DeleteDirectoryOption.DeleteAllContents)
        My.Computer.FileSystem.CreateDirectory(currDir)
        mf.save(currDir & "\manifest.xml")
        fz.ExtractZip(zipPath, currDir, "")
    End Sub
#End Region

End Class
