﻿Imports LangLib
Imports AddOnsLib
Imports System.Windows.Forms

Public Class LangRepoExplorer
    Dim baseDir As String = Application.StartupPath & "\lang\"
    Dim languages As New List(Of String)
    Dim repo As String()()

    Private Sub LangRepoExplorer_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        applyLang(Me)
        refreshInstalledList()
        bw.RunWorkerAsync()
    End Sub

    Sub refreshInstalledList()
        languages.Clear()
        installedList.Items.Clear()
        For Each lPath As String In My.Computer.FileSystem.GetFiles(basedir)
            Dim fInfo As New IO.FileInfo(lPath)
            Dim lName As String = fInfo.Name.Remove(fInfo.Name.Length - 4, 4)
            If lName <> "_def" Then
                Try
                    Dim l As Lang = Lang.load(lPath)
                    languages.Add(lName)
                    installedList.Items.Add(l.fullName)
                Catch ex As Exception
                End Try
            End If
        Next
    End Sub

    Private Sub btnInstall_Click(sender As Object, e As EventArgs) Handles btnInstall.Click
        If repositoryList.SelectedIndex = -1 Then
            MsgBox(getMsg("noLangSelected"), MsgBoxStyle.Critical)
        Else
            Dim list As New List(Of itemToDownload)
            'TODO
            Dim i As Integer = repositoryList.SelectedIndex
            Try
                list.Add(New itemToDownload(RepositoryApi.getSourceforgeDownloadLink(RepositoryApi.getTranslationDownloadLink(My.Application.Info.Version, repo(i)(0))), baseDir & repo(i)(0) & ".xml", repositoryList.SelectedItem & " translation for UniversalIDE"))
            Catch ex As repositoryException
            End Try
            For Each a As addonManifest In MainForm.addons.installedAddons
                Try
                    list.Add(New itemToDownload(RepositoryApi.getSourceforgeDownloadLink(RepositoryApi.getAddonTranslationDownloadLink(a.shortName, a.version, repo(i)(0))), Application.StartupPath & "\Addons\" & a.shortName & "\lang\" & repo(i)(0) & ".xml", repositoryList.SelectedItem & " translation for " & a.fullName))
                Catch ex As repositoryException
                End Try
            Next
            If list.Count = 0 Then
                MsgBox(getMsg("noLangDownload"), MsgBoxStyle.Exclamation)
            Else
                DownloaderTool.download(list)
                refreshInstalledList()
            End If
        End If
    End Sub

    Private Sub btnUninstall_Click(sender As Object, e As EventArgs) Handles btnUninstall.Click
        If installedList.SelectedIndex = -1 Then
            MsgBox(getMsg("noLangSelected"), MsgBoxStyle.Critical)
        Else
            If MainForm.mainSettings.getItemValue("lang") = languages(installedList.SelectedIndex) Then
                MsgBox(getMsg("cannotUninstallCurrentLang"), MsgBoxStyle.Exclamation)
            ElseIf MsgBox(getMsg("uninstallLangConfirm", installedList.SelectedValue), MsgBoxStyle.YesNo) = MsgBoxResult.Yes Then
                My.Computer.FileSystem.DeleteFile(Application.StartupPath & "\lang\" & languages(installedList.SelectedIndex) & ".xml")
                refreshInstalledList()
            End If
        End If
    End Sub

    Private Sub btnCancel_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCancel.Click
        Me.DialogResult = System.Windows.Forms.DialogResult.Cancel
        Me.Close()
    End Sub

    Private Sub bw_DoWork(sender As Object, e As System.ComponentModel.DoWorkEventArgs) Handles bw.DoWork
        Try
            repo = RepositoryApi.getTranslations()
            e.Result = ""
        Catch ex As Exception
            e.Result = ex.Message
        End Try
    End Sub

    Private Sub bw_RunWorkerCompleted(sender As Object, e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles bw.RunWorkerCompleted
        If e.Result = "" Then
            For Each t As String() In repo
                repositoryList.Items.Add(t(1))
            Next
            progress.Visible = False
        Else
            MsgBox(e.Result, MsgBoxStyle.Critical)
            Me.DialogResult = System.Windows.Forms.DialogResult.Cancel
            Me.Close()
        End If
    End Sub
End Class
