Imports AddOnsLib
Imports LangLib
Imports Microsoft.Win32
Imports System.Threading
Imports System.Security.Principal

Public Class MainForm

    Dim currProject As Project
    Public Shared isPortable As Boolean = True
    Public Shared isAdministrator As Boolean = False
    Public Shared settingsPath As String = ""
    Public Shared addons As AddonManager
    Public Shared mainSettings As GenericConfig
    Dim addonsUpdatesString As String = ""
    Dim universalIDEupdate As Version

#Region "Initialization"
    Private Sub MainForm_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        checkPortable()
        checkAdministrator()
        loadSettings()
        If (isPortable Or isAdministrator) And Not My.Computer.FileSystem.DirectoryExists(Application.StartupPath & "\Addons") Then My.Computer.FileSystem.CreateDirectory(Application.StartupPath & "\Addons")
        If isPortable And My.Computer.FileSystem.FileExists(Application.StartupPath & "\UniversalIDE_old.exe") Then My.Computer.FileSystem.DeleteFile(Application.StartupPath & "\UniversalIDE_old.exe")
        If isPortable And My.Computer.FileSystem.FileExists(Application.StartupPath & "\LangLib_old.dll") Then My.Computer.FileSystem.DeleteFile(Application.StartupPath & "\LangLib_old.dll")
        If isPortable And My.Computer.FileSystem.FileExists(Application.StartupPath & "\AddOnsLib_old.dll") Then My.Computer.FileSystem.DeleteFile(Application.StartupPath & "\AddOnsLib_old.dll")
        If isPortable And My.Computer.FileSystem.FileExists(Application.StartupPath & "\ICSharpCode.SharpZipLib_old.dll") Then My.Computer.FileSystem.DeleteFile(Application.StartupPath & "\ICSharpCode.SharpZipLib_old.dll")
        If isPortable And My.Computer.FileSystem.FileExists(Application.StartupPath & "\update.zip") Then My.Computer.FileSystem.DeleteFile(Application.StartupPath & "\update.zip")
        Dim uninstallListFile As String = Application.StartupPath & "\Addons\uninstall.txt"
        If My.Computer.FileSystem.FileExists(uninstallListFile) Then
            Dim uninstallList() As String = My.Computer.FileSystem.ReadAllText(uninstallListFile).Split(Environment.NewLine)
            For Each i As String In uninstallList
                i = i.Replace(vbLf, "")
                My.Computer.FileSystem.DeleteDirectory(Application.StartupPath & "\Addons\" & i, FileIO.DeleteDirectoryOption.DeleteAllContents)
                My.Computer.FileSystem.DeleteDirectory(settingsPath & "\" & i, FileIO.DeleteDirectoryOption.DeleteAllContents)
            Next
            My.Computer.FileSystem.DeleteFile(uninstallListFile)
        End If
        Tabs.TabPages.Clear()
        StartPage.Tag = New tabInfo(tabType.Home, "", Nothing)
        Tabs.TabPages.Add(StartPage)
        UpdateTabs()
        UpdateRecentProjects()
        Dim noAddons As Boolean = My.Application.CommandLineArgs.Contains("-noaddons")
        addons = New AddonManager(noAddons)
        If noAddons Then
            Dim m As New AddonManagerDialog
            m.ShowDialog()
        End If
        StatusText.Text = getMsg("statusReady")
        Dim updaterThread As New Thread(Sub()
                                            Try
                                                universalIDEupdate = RepositoryApi.checkUniversalIDEUpdates()
                                                Dim addonsUpdates() As String = RepositoryApi.checkAddonsUpdates(addons.installedAddons)
                                                For Each i As String In addonsUpdates
                                                    addonsUpdatesString &= "- " & i & Environment.NewLine
                                                Next
                                            Catch ex As Exception
                                            End Try
                                        End Sub)
        updaterThread.Start()
        UpdaterTimer.Start()
        Control.CheckForIllegalCrossThreadCalls = False
        If My.Application.CommandLineArgs.Count > 0 AndAlso My.Application.CommandLineArgs(0) <> "-noaddons" Then
            Try
                OpenPrj(My.Application.CommandLineArgs(0))
            Catch ex As Exception
            End Try
        End If
    End Sub

    Private Sub UpdaterTimer_Tick(sender As Object, e As EventArgs) Handles UpdaterTimer.Tick
        UpdaterTimer.Stop()
        If addonsUpdatesString <> "" AndAlso MessageBox.Show(getMsg("addonUpdatesAvaiable", Environment.NewLine & addonsUpdatesString), "UniversalIDE", MessageBoxButtons.YesNo, MessageBoxIcon.Information) = Windows.Forms.DialogResult.Yes Then
            UpdaterTimer.Stop()
            Dim m As New AddonManagerDialog
            m.ShowDialog()
        ElseIf Not IsNothing(universalIDEupdate) Then
            Dim u As New updateDialog(universalIDEupdate)
            u.ShowDialog()
        End If
    End Sub

    Sub checkPortable()
        Dim regKeyInstall As RegistryKey = Registry.LocalMachine.OpenSubKey("SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall")
        If checkSubKeysForPortable(regKeyInstall) Then
            isPortable = False
            Exit Sub
        End If
        regKeyInstall = Registry.LocalMachine.OpenSubKey("SOFTWARE\Wow6432Node\Microsoft\Windows\CurrentVersion\Uninstall")
        If checkSubKeysForPortable(regKeyInstall) Then isPortable = False
    End Sub

    Function checkSubKeysForPortable(ByVal regKey As RegistryKey) As Boolean
        For Each s As String In regKey.GetSubKeyNames
            If regKey.OpenSubKey(s).GetValue("InstallLocation") = Application.StartupPath & "\" Then
                Return True
            End If
        Next
    End Function

    Sub checkAdministrator()
        Dim wp As New WindowsPrincipal(WindowsIdentity.GetCurrent())
        isAdministrator = wp.IsInRole("Administrators")
    End Sub

    Sub loadSettings()
        If isPortable Then
            settingsPath = Application.StartupPath & "\config"
        Else
            settingsPath = My.Computer.FileSystem.GetParentPath(Application.UserAppDataPath)
        End If
        If Not My.Computer.FileSystem.FileExists(settingsPath & "\UniversalIDE.config.xml") Then
            My.Computer.FileSystem.CopyFile(Application.StartupPath & "\default.config.xml", settingsPath & "\UniversalIDE.config.xml", True)
        End If
        mainSettings = GenericConfig.load(settingsPath & "\UniversalIDE.config.xml")
        loadLang()
    End Sub

    Sub loadLang()
        LangUtil.loadFiles(Application.StartupPath & "\lang\" & mainSettings.getItemValue("lang") & ".xml", Application.StartupPath() & "\lang\_def.xml")
        applyLang(Me, TabsContextMenu)
        Me.Text = My.Application.Info.Title & " " & My.Application.Info.Version.Major & "." & My.Application.Info.Version.Minor
    End Sub
#End Region

#Region "Menus and toolbars"

    Private Sub exitprogram_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles exitprogram.Click
        Application.Exit()
    End Sub

    Private Sub about_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles about.Click, about_.Click
        MsgBox("UniversalIDE, open source multi-language developing environment for windows, version " & My.Application.Info.Version.ToString(), MsgBoxStyle.Information)
    End Sub

    Private Sub closetab_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles closetab_.Click, closetab.Click
        tabClose(Tabs.SelectedTab)
    End Sub

    Private Sub closeexcept_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles closeexcept_.Click, closeexcept.Click
        Dim cs As TabPage = Tabs.SelectedTab
        For Each s As TabPage In Tabs.TabPages
            tabClose(s)
        Next
        Tabs.TabPages.Add(cs)
    End Sub

    Private Sub openstartpage_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles openstartpage_.Click, openstartpage.Click
        If Tabs.Contains(StartPage) = False Then
            Tabs.TabPages.Add(StartPage)
        End If
        Tabs.SelectedTab = StartPage
        UpdateTabs()
        UpdateRecentProjects()
    End Sub

    Private Sub openrecent_DropDownItemClicked(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ToolStripItemClickedEventArgs) Handles openrecent.DropDownItemClicked
        OpenPrj(e.ClickedItem.Text)
    End Sub

    Private Sub savecurrent_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles savecurrent.Click, savecurrent_.Click
        saveTab(Tabs.SelectedIndex)
    End Sub

    Private Sub saveall_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles saveall.Click
        For i As Integer = 0 To Tabs.TabPages.Count - 1
            If Not Object.Equals(Tabs.TabPages(i), StartPage) Then
                saveTab(i)
            End If
        Next
    End Sub

    Private Sub cut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cut_.Click, cut.Click
        Try
            CType(Tabs.SelectedTab.Controls(0), Editor).cut()
        Catch ex As Exception
        End Try
    End Sub

    Private Sub copy_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles copy_.Click, copy.Click
        Try
            CType(Tabs.SelectedTab.Controls(0), Editor).copy()
        Catch ex As Exception
        End Try
    End Sub

    Private Sub paste_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles paste_.Click, paste.Click
        Try
            CType(Tabs.SelectedTab.Controls(0), Editor).paste()
        Catch ex As Exception
        End Try
    End Sub

    Private Sub undo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles undo.Click
        Try
            CType(Tabs.SelectedTab.Controls(0), Editor).undo()
        Catch ex As Exception
        End Try
    End Sub

    Private Sub redo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles redo.Click
        Try
            CType(Tabs.SelectedTab.Controls(0), Editor).redo()
        Catch ex As Exception
        End Try
    End Sub

    Private Sub selectall_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles selectall.Click
        Try
            CType(Tabs.SelectedTab.Controls(0), Editor).selectAll()
        Catch ex As Exception
        End Try
    End Sub

    Private Sub newapplication_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles newapplication.Click, newapplication_.Click
        newapp()
    End Sub

    Private Sub newfile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles newfile.Click, newfile_.Click
        Try
            Dim fn As String = InputBox(getMsg("name"))
            My.Computer.FileSystem.WriteAllText(currPath() & fn, "", False)
            Dim n As TreeNode = FileManager.SelectedNode
            If n.ImageIndex <> 1 Then
                Do
                    n = n.Parent
                    If n.ImageIndex = 1 Then
                        Exit Do
                    End If
                Loop
            End If
            Dim fRelPath As String = currPath(True, False) & fn
            currProject.applications(n.Index).AppFiles.Add(New fileDetails(fRelPath, "", False))
        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Critical)
        End Try
        RefreshFiles()
    End Sub

    Private Sub newdir_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles newdir.Click, newdir_.Click
        Dim dn As String = InputBox(getMsg("name"))
        My.Computer.FileSystem.CreateDirectory(currPath() & dn)
        RefreshFiles()
    End Sub

    Private Sub buildrun_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles buildrun_.Click, buildrun.Click
        build()
        run()
    End Sub

    Private Sub onlybuild_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles onlybuild.Click
        build()
    End Sub

    Private Sub onlyrun_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles onlyrun.Click
        run()
    End Sub

    Private Sub options_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles options.Click
        Dim settingsForm As New SettingsDialog
        settingsForm.ShowDialog()
        loadSettings()
    End Sub

    Private Sub addonmanagment_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles addonmanagment.Click
        If AdministratorControl.check() Then
            Dim m As New AddonManagerDialog
            m.ShowDialog()
        End If
    End Sub
#End Region

#Region "Tabs"
    Public Function getTabInfo(ByVal tab As TabPage) As tabInfo
        Return CType(tab.Tag, tabInfo)
    End Function

    Private Sub MainSplitContainer_SplitterMoved(ByVal sender As System.Object, ByVal e As System.Windows.Forms.SplitterEventArgs) Handles MainSplitContainer.SplitterMoved
        Tabs.Refresh()
    End Sub

    Sub UpdateTabs()
        window.DropDownItems.Clear()
        TabsContextMenu.Items.Clear()
        window.DropDownItems.Add(closetab)
        window.DropDownItems.Add(closeexcept)
        window.DropDownItems.Add(openstartpage)
        window.DropDownItems.Add(ToolStripSeparator8)
        TabsContextMenu.Items.Add(closetab_)
        TabsContextMenu.Items.Add(closeexcept_)
        TabsContextMenu.Items.Add(openstartpage_)
        TabsContextMenu.Items.Add(ToolStripSeparator9)
        For i As Integer = 0 To Tabs.TabPages.Count - 1
            Dim scheda As TabPage = Tabs.TabPages.Item(i)
            Dim e As New EventHandler(Sub(sender As Object, ea As EventArgs)
                                          Tabs.SelectedIndex = CInt(CType(sender, ToolStripMenuItem).Text.Split(" .")(0)) - 1
                                      End Sub)
            window.DropDownItems.Add(i + 1 & ". " & scheda.Text, Nothing, e)
            TabsContextMenu.Items.Add(i + 1 & ". " & scheda.Text, Nothing, e)
            window.DropDownItems.Item(i + 4).Tag = "notranslation"
            TabsContextMenu.Items.Item(i + 4).Tag = "notranslation"
        Next
    End Sub

    Function tabId(ByVal file As String) As Integer
        For i As Integer = 0 To Tabs.TabPages.Count - 1
            If getTabInfo(Tabs.TabPages(i)).f = file Then
                Return i
            End If
        Next
        Return -1
    End Function

    Sub tabClose(ByVal s As TabPage)
        Dim tInfo As tabInfo = getTabInfo(s)
        If tInfo.t = tabType.Prj Then
            currProject.detailsTabOpened = False
        ElseIf tInfo.t = tabType.File Then
            currProject.applications(tInfo.appId).setFileOpen(tInfo.f, False)
        End If
        Tabs.TabPages.Remove(s)
    End Sub

    Sub openFile(ByVal file As String, ByVal appId As Integer)
        Dim absFile As String = ""
        If file = currProject.File Then
            absFile = file
        Else
            absFile = currProject.dir() & "\src\" & currProject.applications(appId).Name & "\" & file
        End If
        Dim id As Integer = tabId(absFile)
        If id = -1 Then
            Dim fi As New IO.FileInfo(absFile)
            Dim c As Control = Nothing
            Dim t As tabType
            If fi.Extension = ".uidp" Then
                t = tabType.Prj
                c = New ProjectDetails(currProject)
                applylangControl(Me.Name, c)
                currProject.detailsTabOpened = True
            Else
                t = tabType.File
                c = addons.getAppType(currProject.applications(appId).AppType).getDefaultEditor(fi.Extension, addons.getLanguages)
                If Object.Equals(c, Nothing) Then
                    MsgBox(getMsg("noAssociatedEditor"), MsgBoxStyle.Critical)
                    Exit Sub
                Else
                    CType(c, Editor).open(absFile)
                End If
                currProject.applications(appId).setFileOpen(absFile)
            End If
            Dim p As New TabPage(fi.Name)
            c.Dock = DockStyle.Fill
            p.Controls.Add(c)
            p.Tag = New tabInfo(t, absFile, appId)
            Tabs.TabPages.Add(p)
            Tabs.SelectedTab = p
        Else
            Tabs.SelectedIndex = id
        End If
    End Sub

    Sub saveTab(ByVal id As Integer)
        saveProject()
        Dim f As String = getTabInfo(Tabs.TabPages(id)).f
        If f <> Nothing And f <> currProject.File Then
            Dim c As Control = Tabs.TabPages(id).Controls(0)
            Try
                CType(c, Editor).save(f)
            Catch ex As Exception
                MsgBox(ex.Message)
            End Try
        End If
    End Sub
#End Region

#Region "FileManager"

    Public Sub RefreshFiles()
        FileManager.Nodes.Clear()
        FileManager.Nodes.Add(currProject.name, currProject.name, 0)
        For Each app As ProjectApplication In currProject.applications
            Dim n As New TreeNode(app.Name, 1, 1)
            n.Tag = app.AppType
            FileManager.Nodes(0).Nodes.Add(n)
            Dim ok As Boolean = True
            If app.dir = "" Then
                n.Nodes.Add("", "Cannot find directory", 2)
                ok = False
            End If
            If Not addons.exists(itemType.AppType, app.AppType) Then
                n.Nodes.Add("", "Cannot find add-on", 2)
                ok = False
            End If
            If ok Then
                addDir(app.dir, n, app.Name)
            End If
        Next
        FileManager.Nodes(0).Expand()
    End Sub

    Sub addDir(ByVal Cartella As String, ByVal n As TreeNode, ByVal app As String)
        For Each corrdir As String In My.Computer.FileSystem.GetDirectories(Cartella)
            n.Nodes.Add(corrdir, New IO.DirectoryInfo(corrdir).Name, 3)
            addDir(corrdir, n.LastNode, app)
        Next
        For Each file As String In My.Computer.FileSystem.GetFiles(Cartella)
            Dim nd As New TreeNode(New IO.FileInfo(file).Name, 4, 4)
            nd.Tag = file.Remove(0, currProject.dir().Length + ("\src\" & app & "\").Length)
            n.Nodes.Add(nd)
        Next
    End Sub

    Private Sub FileManager_AfterSelect(ByVal sender As System.Object, ByVal e As System.Windows.Forms.TreeViewEventArgs) Handles FileManager.AfterSelect
        e.Node.SelectedImageIndex = e.Node.ImageIndex
    End Sub

    Private Sub FileManager_NodeMouseDoubleClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.TreeNodeMouseClickEventArgs) Handles FileManager.NodeMouseDoubleClick
        If Object.Equals(e.Node.Parent, Nothing) Then
            openProjectDetails()
        ElseIf Object.Equals(e.Node.Parent.Parent, Nothing) Then
            Try
                addons.getAppType(currProject.applications(e.Node.Index).AppType).editOptions(currProject.applications(e.Node.Index))
            Catch ex As Exception
                MsgBox(ex.Message, MsgBoxStyle.Critical)
                RefreshFiles()
            End Try
        ElseIf e.Node.ImageIndex <> 3 Then
            Dim n As TreeNode = e.Node
            Do
                n = n.Parent
                If n.ImageIndex = 1 Then
                    Exit Do
                End If
            Loop
            openFile(e.Node.Tag, n.Index)
        End If
        UpdateTabs()
    End Sub

    Function currPath(Optional ByVal rel As Boolean = False, Optional ByVal includeApp As Boolean = True) As String
        If Not rel And Not includeApp Then Throw New ArgumentException()
        Dim n As TreeNode = FileManager.SelectedNode
        Dim appRoot As String = currProject.dir & "\src\"
        If rel Then
            appRoot = ""
        End If
        If Object.Equals(n, Nothing) Then
            Throw New ArgumentException(getMsg("noSelectedDir"))
        End If
        If n.ImageIndex < 3 Then
            If n.ImageIndex = 1 Then
                If includeApp Then
                    Return appRoot & n.Text & "\"
                Else
                    Return ""
                End If
            Else
                Throw New ArgumentException(getMsg("noDirAccessible"))
            End If
        Else
            If n.ImageIndex <> 3 Then
                n = n.Parent
            End If
            Dim p As String = ""
            Do
                p = n.Text & "\" & p
                n = n.Parent
            Loop Until n.ImageIndex = 1
            p = appRoot & n.Text & "\" & p
            If Not includeApp Then
                p = p.Remove(0, n.Text.Length + 1)
            End If
            Return p
        End If
    End Function
#End Region

#Region "Home"
    Private Sub newproject_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles newproject.Click, newproject_.Click, btnHomeNew.Click
        Dim np As New NewPrjDialog
        If np.ShowDialog() = Windows.Forms.DialogResult.OK Then
            OpenPrj(np.f)
        End If
    End Sub

    Private Sub HomeRecent_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles HomeRecent.SelectedIndexChanged
        OpenPrj(HomeRecent.SelectedItem)
    End Sub

    Private Sub openproject_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles openproject.Click, openproject_.Click, btnHomeOpen.Click
        If OpenDialog.ShowDialog() = Windows.Forms.DialogResult.OK Then
            OpenPrj(OpenDialog.FileName)
        End If
    End Sub
#End Region

#Region "Projects"

    Sub OpenPrj(ByVal file As String)
        Dim ok As Boolean = False
        If IsNothing(currProject) Then
            ok = True
        Else
            ok = (MsgBox(getMsg("closeProjectAlert"), MsgBoxStyle.YesNo) = MsgBoxResult.Yes)
        End If
        If ok Then
            If My.Computer.FileSystem.FileExists(file) Then
                StatusText.Text = getMsg("openingProject", file)
                Try
                    currProject = Project.Open(file)
                Catch ex As Exception
                    MsgBox(ex.Message, MsgBoxStyle.Critical)
                    StatusText.Text = getMsg("statusReady")
                    Exit Sub
                End Try
                RefreshFiles()
                For Each s As TabPage In Tabs.TabPages
                    If s.Text <> StartPage.Text Then
                        Tabs.TabPages.Remove(s)
                    End If
                Next
                If mainSettings.getItem("recentProjects").subItemsValues.Contains(file) Then
                    mainSettings.getItem("recentProjects").removeSubItem(file)
                End If
                mainSettings.getItem("recentProjects").subItems.Add(New serializableItem("", file))
                mainSettings.Save()
                UpdateRecentProjects()
                If currProject.detailsTabOpened Then
                    openProjectDetails()
                End If
                For appId As Integer = 0 To currProject.applications.Count - 1
                    For Each f As fileDetails In currProject.applications(appId).AppFiles
                        If f.opened Then
                            openFile(f.p, appId)
                        End If
                    Next
                Next
                StatusText.Text = getMsg("statusReady")
            Else
                MsgBox(getMsg("cannotFindFile", file), MsgBoxStyle.Critical)
                If mainSettings.getItem("recentProjects").subItemsValues.Contains(file) Then
                    If MsgBox(getMsg("removeFromRecent"), MsgBoxStyle.YesNo) = MsgBoxResult.Yes Then
                        mainSettings.getItem("recentProjects").removeSubItem(file)
                        UpdateRecentProjects()
                    End If
                End If
            End If
        End If
    End Sub

    Sub saveProject()
        currProject.Save(Tabs)
    End Sub

    Sub UpdateRecentProjects()
        HomeRecent.Items.Clear()
        openrecent.DropDownItems.Clear()
        For i As Integer = mainSettings.getItem("recentProjects").subItems.Count - 1 To 0 Step -1
            HomeRecent.Items.Add(mainSettings.getItem("recentProjects").subItemsValues(i))
            If Math.Abs(-i) >= mainSettings.getItem("recentProjects").subItems.Count - 5 Then
                Dim tsi As New ToolStripMenuItem(mainSettings.getItem("recentProjects").subItemsValues(i))
                tsi.Tag = "notranslation"
                openrecent.DropDownItems.Add(tsi)
            End If
        Next
    End Sub

    Sub openProjectDetails()
        openFile(currProject.File, Nothing)
    End Sub

    Sub newapp()
        Dim f As New NewAppDialog
        If f.ShowDialog() = Windows.Forms.DialogResult.OK Then
            'If MsgBox(getMsg("needSave"), MsgBoxStyle.YesNo) = MsgBoxResult.Yes Then
            currProject.applications.Add(New ProjectApplication(f.appName.Text, addons.getAppTypes()(f.appType.SelectedIndex), currProject.dir()))
            saveProject()
            RefreshFiles()
            'End If
        End If
    End Sub
#End Region

#Region "Debug"
    Sub build()
        Dim o As New BuildingOutput
        AddHandler o.log.actionAdded, Sub(time As DateTime, action As String)
                                          OutputTextBox.AppendText(String.Format("[{0}] {1}", time.ToLongTimeString, action & Environment.NewLine))
                                          ErrorList.Items.Clear()
                                          If Not o.ok Then
                                              For Each e As buildError In o.errors
                                                  ErrorList.Items.Add(New ListViewItem({"appName", e.descr, e.file.p, e.line, e.column}))
                                              Next
                                          End If
                                      End Sub
        Dim buildTh As New Thread(Sub()
                                      OutputTextBox.AppendText(String.Format("[{0}] Begin building project {1}...{2}", DateTime.Now.ToLongTimeString, currProject.name, Environment.NewLine))
                                      For Each app As ProjectApplication In currProject.applications
                                          'OutputTextBox.AppendText()
                                          Dim d As String = currProject.dir & "\bin\" & app.Name
                                          If Not My.Computer.FileSystem.DirectoryExists(d) Then
                                              My.Computer.FileSystem.CreateDirectory(d)
                                          End If
                                          Try
                                              OutputTextBox.AppendText(String.Format("[{0}] Begin building application {1}...{2}", DateTime.Now.ToLongTimeString, app.Name, Environment.NewLine))
                                              addons.getAppType(app.AppType).build(app, d, o)
                                              Dim errCount As Integer = 0
                                              If Not o.ok Then errCount = o.errors.Length
                                              OutputTextBox.AppendText(String.Format("[{0}] Finished building application {1}, {2} Error(s){3}", DateTime.Now.ToLongTimeString, app.Name, errCount, Environment.NewLine))
                                          Catch ex As Exception
                                              MsgBox(ex.Message, MsgBoxStyle.Critical)
                                          End Try
                                      Next
                                      OutputTextBox.AppendText(String.Format("[{0}] Finished building project.{1}", DateTime.Now.ToLongTimeString, Environment.NewLine))
                                      'My.Computer.FileSystem.WriteAllText(currProject.dir & "\LastBuildLog.txt", log, False)
                                      'Process.Start(currProject.dir & "\LastBuildLog.txt")
                                  End Sub)
        buildTh.Start()
    End Sub

    Sub run()
        For Each app As ProjectApplication In currProject.applications
            Dim d As String = ""
            Dim t As ApplicationType = addons.getAppType(app.AppType)
            If t.isParsed Then
                d = currProject.dir & "\src\" & app.Name
            Else
                d = currProject.dir & "\bin\" & app.Name
            End If
            t.run(d)
        Next
    End Sub
#End Region

End Class
