﻿Imports LangLib
Imports System.Threading
Imports System.Windows.Forms

Public Class RepoExplorer
    Public Shared api As New RepositoryApi
    Dim addons()() As String
    Public selectedAddonMF As addonManifest

    Private Function getFullName(ByVal shortName As String) As String
        For Each i As String() In addons
            If i(0) = shortName Then Return i(1).Remove(0, 1)
        Next
        Return Nothing
    End Function

    Private Sub RepoExplorer_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        applyLang(Me)
        progress.Visible = True
        bw.RunWorkerAsync(bwAction.getList)
    End Sub

    Private Sub addonList_SelectedIndexChanged(sender As Object, e As EventArgs) Handles addonList.SelectedIndexChanged
        lblSelect.Visible = False
        FlowLayout.Dock = DockStyle.Fill
        versionSelector.Items.Clear()
        For Each s As String In RepositoryApi.getAddonVersions(addons(addonList.SelectedIndex)(0))
            versionSelector.Items.Add(s)
        Next
        versionSelector.SelectedIndex = 0
        loadDetails()
    End Sub

    Sub loadDetails()
        FlowLayout.Visible = True
        selectedAddonMF = RepositoryApi.getAddonManifest(addons(addonList.SelectedIndex)(0), versionSelector.SelectedIndex)
        lblAddonName.Text = selectedAddonMF.fullName
        lblAddonDev.Text = "by " + selectedAddonMF.developerName
        lblAddonDev.LinkArea = Nothing
        lblAddonDev.LinkArea = New LinkArea(3, selectedAddonMF.developerName.Length)
        lblAddonDev.Links(0).LinkData = RepositoryApi.getDeveloperLink(selectedAddonMF.developerID)
        If MainForm.addons.installedAddons.Contains(selectedAddonMF) Then
            lblInstalled.Visible = True
            Dim selectedVersion As Version = selectedAddonMF.getVersion()
            Dim installedversion As Version = MainForm.addons.installedAddons.Find(Function(obj As addonManifest) As Boolean
                                                                                       Return obj.shortName = selectedAddonMF.shortName
                                                                                   End Function).getVersion()
            If selectedVersion > installedversion Then
                lblInstalled.Text = getMsg("newerVersionSelected")
            ElseIf selectedVersion < installedversion Then
                lblInstalled.Text = getMsg("olderVersionSelected")
            ElseIf selectedVersion = installedversion Then
                lblInstalled.Text = getMsg("sameVersionSelected")
            End If
        End If
        Dim comp As compatibilityLevel = selectedAddonMF.checkCompatibility()
        lblCompatibility.Text = getMsg("compatibilityLevel." & comp.ToString())
        Select Case comp
            Case compatibilityLevel.yes
                lblCompatibility.ForeColor = Color.Green
            Case compatibilityLevel.no
                lblCompatibility.ForeColor = Color.Red
            Case compatibilityLevel.unknown
                lblCompatibility.ForeColor = Color.FromArgb(255, 162, 0)
        End Select
        lblAddonDesc.Text = selectedAddonMF.description
        lblAddonDep.Text = ""
        lblAddonDep.Text = addonDependancySolver.solveDependancies(selectedAddonMF).s
        If lblAddonDep.Text = "" Then lblAddonDep.Text = "None"
    End Sub

    Private Sub versionSelector_SelectedIndexChanged(sender As Object, e As EventArgs) Handles versionSelector.SelectedIndexChanged
        loadDetails()
    End Sub

    Private Sub bw_DoWork(sender As Object, e As System.ComponentModel.DoWorkEventArgs) Handles bw.DoWork
        Select Case e.Argument
            Case bwAction.getList
                Try
                    addons = RepositoryApi.getAddons()
                    e.Result = ""
                Catch ex As Exception
                    e.Result = ex.Message
                End Try
        End Select
    End Sub

    Private Sub bw_RunWorkerCompleted(sender As Object, e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles bw.RunWorkerCompleted
        If e.Result <> "" Then
            MsgBox(e.Result, MsgBoxStyle.Critical)
            Me.DialogResult = System.Windows.Forms.DialogResult.Cancel
            Me.Close()
        Else
            For Each a As String() In addons
                addonList.Items.Add(a(1))
            Next
        End If
        progress.Visible = False
    End Sub

    Private Sub lblAddonDev_LinkClicked(sender As Object, e As LinkLabelLinkClickedEventArgs) Handles lblAddonDev.LinkClicked
        Process.Start(e.Link.LinkData)
    End Sub

    Private Sub btnOk_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOk.Click
        Me.DialogResult = System.Windows.Forms.DialogResult.OK
        Me.Close()
    End Sub

    Private Sub btnCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCancel.Click
        Me.DialogResult = System.Windows.Forms.DialogResult.Cancel
        Me.Close()
    End Sub

    Enum bwAction
        getList
        getManifest
    End Enum
End Class
