﻿Imports System.Net
Imports System.Xml
Imports System.Text.RegularExpressions

Public Class RepositoryApi
    Public Const baseUrl = "http://universalide.sourceforge.net"

#Region "Misc utilities"
    Private Shared Function getList(ByVal url As String)
        Dim w As New WebClient()
        Dim s As String = w.DownloadString(baseUrl & url)
        Dim ls() As String = s.Split(Environment.NewLine)
        Dim result As New List(Of String())
        For i As Integer = 0 To ls.Length - 2 Step 2
            Dim sn As String = ls(i)
            Dim fn As String = ls(i + 1)
            If i <> 0 Then sn = sn.Remove(0, 1)
            fn = fn.Remove(0, 1)
            result.Add({sn, fn})
        Next
        Return result.ToArray()
    End Function

    Public Shared Function getSourceforgeDownloadLink(ByVal SFpage As String) As String
        Dim w As New WebClient()
        w.Headers("User-Agent") = "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.11 (KHTML, like Gecko) Chrome/17.0.963.56 Safari/535.11"
        Dim html As String = w.DownloadString(SFpage)
        If html = "Error - Invalid querystring" Or html = "Already included" Or html = "Error - not avaiable for download" Then Throw New repositoryException(html)
        Dim re As New Regex("<a href=""(.*)"" class=""direct-download"">")
        Return re.Match(html).Groups(1).Value
    End Function

    Public Shared Function checkUniversalIDEUpdates() As Version
        Dim w As New WebClient
        Dim lastVersion As Version = Version.Parse(w.DownloadString(baseUrl & "/api/getLastVersion.php"))
        If lastVersion > My.Application.Info.Version Then
            Return lastVersion
        Else
            Return Nothing
        End If
    End Function

    Public Shared Function getLastUniversalIDEdownloadLink(ByVal portable As Boolean)
        Dim type As String = "installerBin"
        If portable Then type = "portableBin"
        Return baseUrl & "/dl.php?item=main&type=" & type
    End Function
#End Region

#Region "Addons"
    Public Shared Function getAddons() As String()()
        Return getList("/api/getAddonList.php")
    End Function

    Public Shared Function getFullNames() As Dictionary(Of String, String)
        Dim names As String()() = getAddons()
        Dim result As New Dictionary(Of String, String)
        For i As Integer = 0 To names.Length - 1
            result.Add(names(i)(0), names(i)(1))
        Next
        Return result
    End Function

    Public Shared Function getAddonVersions(ByVal shortName As String) As String()
        Dim w As New WebClient()
        Dim s As String = w.DownloadString(baseUrl & "/api/getAddonVersions.php?id=" & shortName)
        s = s.Remove(s.Length - 2)
        Return s.Split(Environment.NewLine)
    End Function

    Public Shared Function getAddonManifest(ByVal shortName As String, Optional ByVal version As Integer = -1) As addonManifest
        Dim vParam As String = ""
        If version <> -1 Then vParam = "&v=" & version
        Dim w As New WebClient()
        Dim s As String = w.DownloadString(baseUrl & "/api/getAddonManifest.php?id=" & shortName & vParam)
        Return addonManifest.loadString(s)
    End Function

    Public Shared Function getAddonDownloadLink(ByVal shortName As String, ByVal version As String) As String
        Return baseUrl & "/dl.php?item=addon&id=" & shortName & "&v=" & version
    End Function

    Public Shared Function checkAddonsUpdates(ByVal addons As List(Of addonManifest)) As String()
        Dim w As New WebClient
        Dim requestUrl As String = baseUrl & "/api/checkAddonUpdates.php?"
        For i As Integer = 0 To addons.Count - 1
            requestUrl &= String.Format("id{0}={1}&v{0}={2}&", i, addons(i).shortName, addons(i).version)
        Next
        requestUrl = requestUrl.Remove(requestUrl.Length - 1, 1)
        Dim s As String = w.DownloadString(requestUrl)
        If s = "" Then
            Return Nothing
        Else
            Return s.Split(Environment.NewLine.ToCharArray(), StringSplitOptions.RemoveEmptyEntries)
        End If
    End Function
#End Region

#Region "Translations"
    Public Shared Function getTranslationDownloadLink(ByVal version As Version, ByVal langID As String) As String
        Return baseUrl & "/dl.php?item=lang&id=" & langID & "&v=" & version.ToString()
    End Function

    Public Shared Function getAddonTranslationDownloadLink(ByVal addonShortName As String, ByVal version As String, ByVal langID As String) As String
        Return baseUrl & "/dl.php?item=lang&id=" & langID & "&v=" & version & "&for=" & addonShortName
    End Function

    Public Shared Function getTranslations() As String()()
        Return getList("/api/getLangList.php")
    End Function

    Public Shared Function getDeveloperLink(ByVal devName As String) As String
        Return baseUrl & "/repository/developerdetails.php?id=" & devName
    End Function

    Public Shared Function getAvaiableTranslations(ByVal v As Version) As String()
        Dim w As New WebClient
        Return w.DownloadString(baseUrl & "/api/getAvaiableLangs.php?v=" & v.ToString()).Split(",")
    End Function
#End Region

End Class

Public Class repositoryException
    Inherits Exception
    Public exceptionType As repositoryExceptionType

    Sub New(ByVal serverMessage As String)
        Select Case serverMessage
            Case "Error - Invalid querystring"
                exceptionType = repositoryExceptionType.invalidQueryString
            Case "Already included"
                exceptionType = repositoryExceptionType.alreadyIncluded
            Case "Error - not avaiable for download"
                exceptionType = repositoryExceptionType.notAvaiable
        End Select
    End Sub

    Enum repositoryExceptionType
        invalidQueryString
        alreadyIncluded
        notAvaiable
    End Enum
End Class