﻿Imports LangLib
Imports AddOnsLib
Imports ICSharpCode.SharpZipLib.Zip
Imports System.Windows.Forms

Public Class updateDialog
    Dim portable As String = ""
    Dim lastVer As Version

    Sub New(ByVal v As Version)
        lastVer = v
        If MainForm.isPortable Then portable = " (Portable)"
        InitializeComponent()
    End Sub

    Private Sub updateDialog_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        applyLang(Me)
        lblUpdate.Text = String.Format(lblUpdate.Text, My.Application.Info.Version.ToString(), lastVer.ToString())
        lblDownloadAndInstall.Text = String.Format(lblDownloadAndInstall.Text, portable)
        wb.Navigate(RepositoryApi.baseUrl & String.Format("/api/getChangelog.php?from={0}&to={1}", My.Application.Info.Version.ToString(), lastVer.ToString()))
    End Sub

    Private Sub btnOk_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOk.Click
        If Not AdministratorControl.check() Then Exit Sub
        Dim incompatibleAddons As New List(Of String)
        Dim unknownCompatibilityAddons As New List(Of String)
        For Each a As addonManifest In MainForm.addons.installedAddons
            Dim comp As compatibilityLevel = a.universalideCompatibility.Find(New Predicate(Of Compatibility)(Function(obj As Compatibility) As Boolean
                                                                                                                  Return obj.getVersion = lastVer
                                                                                                              End Function)).lv
            If comp = compatibilityLevel.no Then incompatibleAddons.Add(a.fullName)
            If comp = compatibilityLevel.unknown Then unknownCompatibilityAddons.Add(a.fullName)
        Next
        If incompatibleAddons.Count > 0 Or unknownCompatibilityAddons.Count > 0 Then
            Dim msg As String = ""
            If incompatibleAddons.Count > 0 Then
                Dim s As String = Environment.NewLine
                For Each i As String In incompatibleAddons
                    s &= i & Environment.NewLine
                Next
                msg &= getMsg("updateAddonsIncompatible", s)
            End If
            If unknownCompatibilityAddons.Count > 0 Then
                Dim s As String = Environment.NewLine
                For Each i As String In unknownCompatibilityAddons
                    s &= "- " & i & Environment.NewLine
                Next
                msg &= getMsg("updateAddonsUnknownCompatibility", s)
            End If
            msg &= getMsg("updateAddonsInstallAnyway")
            If MessageBox.Show(msg, "UniversalIDE", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) = Windows.Forms.DialogResult.No Then Exit Sub
        End If
        Dim dl As New List(Of itemToDownload)
        Dim updatePath As String = Application.StartupPath & "\update."
        If MainForm.isPortable Then
            updatePath &= "zip"
        Else
            updatePath &= "exe"
        End If
        If My.Computer.FileSystem.FileExists(updatePath) Then My.Computer.FileSystem.DeleteFile(updatePath)
        Dim avaiableLangs As New List(Of String)(RepositoryApi.getAvaiableTranslations(lastVer))
        Dim unavaiableLangs As New List(Of String)
        For Each lng As String In My.Computer.FileSystem.GetFiles(Application.StartupPath & "\lang\")
            Dim lngName As String = New IO.FileInfo(lng).Name
            lngName = lngName.Remove(lngName.Length - 4, 4)
            If lngName <> "_def" AndAlso lngName <> "en" AndAlso Not avaiableLangs.Contains(lngName) Then unavaiableLangs.Add(Lang.load(lng).fullName)
        Next
        If unavaiableLangs.Count > 0 Then
            Dim msg As String = Environment.NewLine
            For Each l As String In unavaiableLangs
                msg &= "- " & l & Environment.NewLine
            Next
            If MessageBox.Show(getMsg("translationsNotAvaiable", msg), "UniversalIDE", MessageBoxButtons.YesNo, MessageBoxIcon.Information) = Windows.Forms.DialogResult.No Then Exit Sub
        End If
        dl.Add(New itemToDownload(RepositoryApi.getSourceforgeDownloadLink(RepositoryApi.getLastUniversalIDEdownloadLink(MainForm.isPortable)), updatePath, "UniversalIDE " & lastVer.ToString() & portable))
        For Each lng As String In My.Computer.FileSystem.GetFiles(Application.StartupPath & "\lang\")
            Dim lngName As String = New IO.FileInfo(lng).Name
            lngName = lngName.Remove(lngName.Length - 4, 4)
            My.Computer.FileSystem.DeleteFile(lng)
            If lngName <> "_def" And lngName <> "en" And avaiableLangs.Contains(lngName) Then
                dl.Add(New itemToDownload(RepositoryApi.getSourceforgeDownloadLink(RepositoryApi.getTranslationDownloadLink(lastVer, lngName)), lng, lngName & " translation for UniversalIDE"))
            End If
        Next
        DownloaderTool.download(dl)
        If MainForm.isPortable Then
            My.Computer.FileSystem.RenameFile(Application.ExecutablePath, "UniversalIDE_old.exe")
            My.Computer.FileSystem.RenameFile(Application.StartupPath & "\LangLib.dll", "LangLib_old.dll")
            My.Computer.FileSystem.RenameFile(Application.StartupPath & "\AddOnsLib.dll", "AddOnsLib_old.dll")
            My.Computer.FileSystem.RenameFile(Application.StartupPath & "\ICSharpCode.SharpZipLib.dll", "ICSharpCode.SharpZipLib_old.dll")
            Dim fz As New FastZip
            fz.ExtractZip(updatePath, Application.StartupPath, "")
            Process.Start(Application.StartupPath & "\UniversalIDE.exe")
            Application.Exit()
        Else
            Process.Start(updatePath, "/SILENT")
            Application.Exit()
        End If
        Me.DialogResult = System.Windows.Forms.DialogResult.OK
        Me.Close()
    End Sub

    Private Sub btnCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCancel.Click
        Me.DialogResult = System.Windows.Forms.DialogResult.Cancel
        Me.Close()
    End Sub
End Class
